/*
-----------------------------------------------------------------------------------
 Project: Alarm Clock by ABEtronics
 By: Jim Abraham
 Date: 11-4-22
 Rev Date: 1-23-23
 Fetures:
 Touch ILI9341 3.2" 320x240 TFT LCD Screen,  RTC, MP3 player, internet connection.
-----------------------------------------------------------------------------------
*/

#include <Arduino.h>
#define ARDUINO_MEGA_2560
#include "SoftwareSerial.h"
#include <BY8001.h>

//---------------------------------------------------------------------
// Baudrate
//---------------------------------------------------------------------
#define SERIAL_BAUDRATE   115200
#define SERIAL1_BAUDRATE  115200

//---------------------------------------------------------------------
// RTC DS3231
//---------------------------------------------------------------------
#include <Wire.h>
#include <ds3231.h>

//---------------------------------------------------------------------
// Variables
//---------------------------------------------------------------------
struct ts t; 
String ampm, tempval="";
String t_hr="", t_min="", t_sec="", t_ampm="", t_mon="", t_yr="", t_day="", t_dow="";
char inchar;
int tampm;
int strt, sett;
int incomingByte = 0; // for incoming serial data
int loopcnt=0;
int startup_min15_1=1 ,startup_min15_2=1, startup_min15_3=1, startup_min15_4=1;
int startup_srt=1, startup_sst=1, startup_RTC=1;
int startup_time1=1, startup_time2=1, startup_time3=1, startup_time4=1;
int min15PA=0, min15FL=0, min15KY=0, min15PAW=0;
int srt=0, sst=0, stime1=0, stime2=0, stime3=0, stime4=0,noTimeUpdate=0;
int lp=0, i, min15up=0,min15uptime=0,min10uptime=0,SM_Closed=0;
int DS3231=0;
int Dlay=4000;
int stmeHR=2; // update time 2am

int Display_Stocks=0; //touchscreen
int min10Stock=0,touchit=0,tmin=0;
int Curtemp=0;
int Curtemp1=0,Curtemp2=0;

//---------------------------------------------------------------------
// TFT LCD Screen
// 3.2 inch TFT display (HX8357 / ILI9481 controller compatible)
// Resolution: 240H x 320W pixels
// Colors: 262K
//---------------------------------------------------------------------
#include <TFT_HX8357.h>         // Hardware-specific library
TFT_HX8357 tft = TFT_HX8357();  // Invoke custom library
#define TFT_GREY 0x5AEB         // int font color.
#include <URTouch.h>
URTouch  myTouch( 6, 5, 4, 3, 2);
//---------------------------------------------------------------------
SoftwareSerial mp3Serial(15, 14);  // RX3, TX3
BY8001 mp3;  // creating an instance of class BY8001 and call it 'mp3'

//========================================================================
// SETUP
//========================================================================
void setup(void)
{
  myTouch.InitTouch();
  myTouch.setPrecision(PREC_MEDIUM);
  
  mp3Serial.begin(9600);       // BY8001 set to 9600 baud (required)  
  mp3.setup(mp3Serial);        // tell BY8001 library which serial port to use. 
  delay(2000);
  mp3.setVolume(20);           // 0-30
  mp3.setLoopPlaybackMode(4);  // single play
  byte playbackStatus = mp3.getPlaybackStatus(); // 
  if (playbackStatus==0)
    {
      mp3.playTrackByIndexNumber(85); // Hello
      delay(2000);
      mp3.playTrackByIndexNumber(84); // System Ready
      delay(500);
    }
   
  Serial.begin(115200);   // opens serial port, sets data rate to 115200 bps
  Serial1.begin(115200);  // opens serial port, sets data rate to 115200 bps
  Wire.begin();
//---------------------------------------------------------------------
  tft.init();
  tft.setRotation(1); //landscape.
  tft.fillScreen(TFT_BLACK);
  tft.setTextColor(TFT_WHITE, TFT_BLACK);    // Set color back to white
  tft.setTextSize(0);
  tft.drawString("Info Display by ABEtronics 10-12-22  ver1.10",12,2, 2);// Title
  tft.drawString("---- Weather Forecast ----",80,124, 2);// Title
  tft.setTextColor(TFT_GREEN, TFT_BLACK);    // Set color back to green
  tft.drawString("PA",43,138, 2); 
  tft.drawString("FL",156,138, 2); 
  tft.drawString("KY",268,138, 2); 
  tft.drawString("SUN-RISE:",10,200, 2); 
  tft.drawString("SUN-SET:",10,215, 2);   
  tft.drawString("-- Weather Conditions --",152,200, 2);
  strt=1;
  GetTime();
  DisplayTime();
  delay (500);
  StartUP();
}

//========================================================================
// LOOP
//========================================================================
void loop() 
{
  Check_touchscreen();
  if (Display_Stocks==1){Stock_Prices_Update_10min();}
  GetTime();
  DisplayTime();
  delay (500);
  if (Display_Stocks==0){Weather_Update_15min();}
  GetTime();
  DisplayTime();
  if (Display_Stocks==0){Time_Update_Daily_2am();}
  if (t.min==0 and tmin==0){TimeIs();}
}

//========================================================================
// Routines
//========================================================================
//---------------------------------------------------------------------
// MP3 Routines
// MP3 files:
// 1-60 ok
// 61=70   -9 ,62=80   -18, 63=90   -27, 64=100, -36 (Num>99 sub 36)
// 65=101 ... 88=124
// 74= Have a Good Day
// 75= The Time is 
// 76= AM
// 77= PM
// 78= O-Clock
// 79= No
// 80= Good Morn
// 81= Good After
// 82= Good Even
// 83= Good Night 
// 84= Sys Ready
// 85= Hello
// 86= Degrees
// 87= Link is down
// 88= The Temp is
// Songs: 89-105
//---------------------------------------------------------------------
void TimeIs()
{
  int thour;
  thour=t.hour;
  byte playbackStatus = mp3.getPlaybackStatus(); 
  tmin=1; 
  if (playbackStatus==0)
    {
      if (thour==6 and ampm=="AM")
        {
          mp3.playTrackByIndexNumber(80);  // 6am, good morning
          delay(2000);
        }
      if (thour==12)
        {
          mp3.playTrackByIndexNumber(81);  // noon, good afternoon
          delay(2000);
        }
      if (thour==17)
        {
          mp3.playTrackByIndexNumber(82);  // 5pm, good evening
          delay(2000);
        }     
         
      mp3.playTrackByIndexNumber(75);     // The Time is
      delay(2000);
      if (thour>12){thour=t.hour-12;}
      mp3.playTrackByIndexNumber(thour); // time
      delay(1000);
      mp3.playTrackByIndexNumber(78);     // O-Clock
      delay(1000);
      
      if (t.hour==22)
        {
          delay(1000);
          mp3.playTrackByIndexNumber(83);  // 10pm, good night
          delay(1000);
        } 
      if (thour==6 and ampm=="AM")
        {
          mp3.setVolume(25);                // 0-30
          delay(1000);
          mp3.playTrackByIndexNumber(74);   // 6am, Have a Good Day
          delay(2000);
        }
    }
    mp3.setVolume(20);                // 0-30
}
//---------------------------------------------------------------------
void PA_TempIs()
{
  delay(1000);
  //Curtemp=tempval.toInt();
  Curtemp1=0,Curtemp2=0;
  byte playbackStatus = mp3.getPlaybackStatus();
  if (playbackStatus==0)
    {
      mp3.playTrackByIndexNumber(88);  // temperature is
      delay(2000);
      
      if (Curtemp<20){mp3.playTrackByIndexNumber(Curtemp);}         // PA temperaure
      if (Curtemp>=20 and Curtemp<70)
        {
          Curtemp1 = Curtemp % 10;       // last digit
          Curtemp2 = Curtemp - Curtemp1; // first digit
          mp3.playTrackByIndexNumber(Curtemp2);  // PA temperaure
          delay(1000);
          if(Curtemp1 != 0){mp3.playTrackByIndexNumber(Curtemp1);}  // PA temperaure
        }
      if (Curtemp>=70 and Curtemp<80)
        {
          Curtemp1 = Curtemp % 10;  // last digit
          Curtemp2 = 61;            // first digit
          mp3.playTrackByIndexNumber(Curtemp2);  // PA temperaure
          delay(1000);
          if(Curtemp1 != 0){mp3.playTrackByIndexNumber(Curtemp1);}  // PA temperaure
        }
      if (Curtemp>=80 and Curtemp<90)
        {
          Curtemp1 = Curtemp % 10;      // last digit
          Curtemp2 = 62;                // first digit
          mp3.playTrackByIndexNumber(Curtemp2);  // PA temperaure
          delay(1000);
          if(Curtemp1 != 0){mp3.playTrackByIndexNumber(Curtemp1);}  // PA temperaure
        }
      if (Curtemp>=90 and Curtemp<100)
        {
          Curtemp1 = Curtemp % 10;      // last digit
          Curtemp2 = 63;                // first digit
          mp3.playTrackByIndexNumber(Curtemp2);  // PA temperaure
          delay(1000);
          if(Curtemp1 != 0){mp3.playTrackByIndexNumber(Curtemp1);}  // PA temperaure
        }
       if (Curtemp>=100)
        {
          Curtemp1 = Curtemp % 100;      // last digit
          Curtemp2 = 64;                 // first digit
          mp3.playTrackByIndexNumber(Curtemp2);  // PA temperaure
          delay(1000);
          if(Curtemp1 != 0){mp3.playTrackByIndexNumber(Curtemp1);}  // PA temperaure
        }
    
      delay(1000);
      mp3.playTrackByIndexNumber(86);               // degrees
      delay(250);
    }
}
//---------------------------------------------------------------------
void Check_touchscreen()
//---------------------------------------------------------------------
{
   if (myTouch.dataAvailable())
    {
      myTouch.read();
      int x=myTouch.getX();
      int y=myTouch.getY();
      tft.drawString("touch",10,124, 2);
//      tft.drawNumber(Display_Stocks, 10, 104, 2); 
      if (Display_Stocks==1)
        {
          Display_Stocks=0;
          touchit=1;
          Display_Weather();
          min15PA=0;
          min15FL=0;
          min15KY=0;
          min15PAW=0;
          min15up=0;
          Weather_Update_15min();        
        }    
      else
        {
          if (x>0 and y>0)
          {
            Display_Stocks=1;
            SM_Closed=0;
            Display_Stock_Prices();
          }
        }
  //    tft.drawNumber(x, 10, 45, 2);
  //    tft.drawNumber(y, 10, 70, 2);    
  //    tft.drawChar('X', x, y, 2);
    }
  else
  {
      tft.drawString("       ",10,124, 2);  
  }  
}
//---------------------------------------------------------------------
void Display_Weather()
//---------------------------------------------------------------------
{
  tft.drawString("                                         ", 1,155, 6);
  tft.drawString("     ",43,138, 2); 
  tft.drawString("                                         ",156,138, 2); 
  tft.drawString("                                         ",268,138, 2); 
  tft.drawString("                                                 ",10,200, 2); 
  tft.drawString("                                                 ",10,215, 2); 
  tft.drawString("                                                 ",10,142, 4); 
  tft.drawString("                                                 ",10,162, 4); 
  tft.drawString("                           ",78,182, 2);
  tft.drawString("                           ",78,202, 2);
  tft.setTextColor(TFT_WHITE, TFT_BLACK);
  tft.drawString("---- Weather Forecast ----",80,124, 2);// Title
  tft.setTextColor(TFT_GREEN, TFT_BLACK);    // Set color back to green
  tft.drawString("PA",43,138, 2); 
  tft.drawString("FL",156,138, 2); 
  tft.drawString("KY",268,138, 2); 
  tft.drawString("SUN-RISE:",10,200, 2); 
  tft.drawString("SUN-SET:",10,215, 2);   
  tft.drawString("-- Weather Conditions --",152,200, 2);
}
  
//---------------------------------------------------------------------
void Stock_Prices_Update_10min()
//---------------------------------------------------------------------
{
      if (t.min==0 or t.min==10 or t.min==20 or t.min==30 or t.min==40 or t.min==50)
      {
        if (min10Stock==0)
         {
            if (SM_Closed==0){Display_Stock_Prices();}
         }  
      }
}
//---------------------------------------------------------------------
void Display_Stock_Prices()
//---------------------------------------------------------------------
{
  min10uptime=1;
  tft.setTextColor(TFT_CYAN, TFT_BLACK);
  tft.drawString("-- Stock Price Updates  --    ",80,124, 2);
  tft.drawString("                                         ", 1,155, 6);
  tft.drawString("   ",43,138, 2); 
  tft.drawString("   ",156,138, 2); 
  tft.drawString("   ",268,138, 2); 
  tft.drawString("                               ",10,200, 2); 
  tft.drawString("                               ",10,215, 2); 
  tft.drawString("                           ",152,200, 2);
  tft.drawString("                           ",172,215, 2);     
 
  tft.setTextColor(TFT_GREEN, TFT_BLACK);
  tft.drawString("   Tesla:",10,142, 2); 
  tft.drawString("S&P500: ",10,162, 2); 
  
  tft.setTextColor(TFT_RED, TFT_BLACK);
  if (t.hour>=16 or t.wday==1 or t.wday==7)
    {
      tft.drawString("- Stock Market Closed -",78,182, 2);
      SM_Closed=1;
    }
  else 
    {
      tft.drawString("                              ",78,182, 2);
    }
    
  if (t.hour>3 and t.hour<10 and t.min<30 and t.wday>1 and t.wday<7)
    {
      tft.setTextColor(TFT_GREEN, TFT_BLACK);
      tft.drawString("--  Pre Market Open  --",78,202, 2);
      SM_Closed=1;
    }
     
  else if (t.hour>16 and t.hour<19 and t.wday>1 and t.wday<7)
    {
      tft.setTextColor(TFT_GREEN, TFT_BLACK);
      tft.drawString("----  After Hours  ----",78,202, 2);
      SM_Closed=1;      
    }
    else
    {
      tft.drawString("                            ",78,202, 2);
    }    
  tft.setTextColor(TFT_WHITE, TFT_BLACK);
 
  do 
    {//empty serial buffer, loop until nothing in buffer.
      inchar = Serial1.read(); 
    } 
  while (Serial1.available() > 0);
  tft.drawChar(' ',290,110, 2);
//==============================================================
// send command for data
  Serial1.println("S1");                  
  delay(Dlay);                            //allow web site to respond.
  tempval="";
         
// check for data arrival
  if (Serial1.available() > 0) 
      {
        // read the incoming byte.
        inchar = Serial1.read();                 //#100*
        if (inchar=='#')
           { 
              tft.drawChar('+', 70,142, 2);
              inchar = Serial1.read(); 
              tft.drawChar(inchar, 80,142, 2);
              inchar = Serial1.read(); 
              tft.drawChar(inchar, 90,142, 2);
              inchar = Serial1.read(); 
              tft.drawChar(inchar, 100,142, 2);
              inchar = Serial1.read(); //.
              tft.drawChar(inchar, 110, 142, 2);
              inchar = Serial1.read(); 
              tft.drawChar(inchar, 115,142, 2);
              inchar = Serial1.read(); 
              tft.drawChar(inchar, 125,142, 2);
            }
      }
  else
      {
        // no respnse from web site.
        tft.drawChar('?',290,110, 2);
      } 
//----------------------------------------------------------
delay(2000); //wait for possible data
do 
  {//empty serial buffer, loop until nothing in buffer.
    inchar = Serial1.read(); 
  } 
while (Serial1.available() > 0);   

//==============================================================
// send command for data
      Serial1.println("S2");                  
      delay(Dlay);                            //allow web site to respond.
      tempval="";
       
// check for data arrival
  if (Serial1.available() > 0) 
      {
        // read the incoming byte.
        inchar = Serial1.read();                 //#-10.76%*
        if (inchar=='#')
           { 
              inchar = Serial1.read();          //+ or -
              if (inchar =='-')
                {
                  tft.setTextColor(TFT_RED, TFT_BLACK);
                }
              else
                {
                  tft.setTextColor(TFT_GREEN, TFT_BLACK);  
                }
              tft.drawChar(inchar, 160,142, 2);
              inchar = Serial1.read();          //1
              tft.drawChar(inchar, 170,142, 2);
              inchar = Serial1.read();          //0
              tft.drawChar(inchar, 180,142, 2);
              inchar = Serial1.read();          //.
              tft.drawChar(inchar, 185,142, 2);
              inchar = Serial1.read();          //7
              tft.drawChar(inchar, 195,142, 2);
              inchar = Serial1.read();          //6
              tft.drawChar(inchar, 205,142, 2);
              inchar = Serial1.read();          //%
              if (inchar !='*'){tft.drawChar(inchar, 220,142, 2);}
              tft.setTextColor(TFT_WHITE, TFT_BLACK);
           }
  else
      {
        // no respnse from web site.
        tft.drawChar('?',290,110, 2);
      } 
//----------------------------------------------------------
delay(2000); //wait for possible data
do 
  {//empty serial buffer, loop until nothing in buffer.
    inchar = Serial1.read(); 
  } 
while (Serial1.available() > 0);   
}
//==============================================================
// send command for data
      Serial1.println("S3");                  
      delay(Dlay);                            //allow web site to respond.
      tempval="";
       
// check for data arrival
  if (Serial1.available() > 0) 
      {
        // read the incoming byte.
        inchar = Serial1.read();                 //#100*
        if (inchar=='#')
           { 
              tft.drawChar('+', 70,162, 2);
              inchar = Serial1.read();          //3
              tft.drawChar(inchar, 80,162, 2);
              inchar = Serial1.read();          //,
              tft.drawChar(inchar, 90,162, 2);
              inchar = Serial1.read();          //8
              tft.drawChar(inchar, 95,162, 2);
              inchar = Serial1.read();          //0
              tft.drawChar(inchar, 105,162, 2);
              inchar = Serial1.read();          //0
              tft.drawChar(inchar, 115, 162, 2);
              inchar = Serial1.read();          //.
              tft.drawChar(inchar, 125,162, 2);
              inchar = Serial1.read();          //00
              tft.drawChar(inchar, 130,162, 2);
              inchar = Serial1.read();          //00
              tft.drawChar(inchar, 140,162, 2);
            }
      }
  else
      {
        // no respnse from web site.
        tft.drawChar('?',290,110, 2);
      } 
//----------------------------------------------------------
delay(2000); //wait for possible data
do 
  {//empty serial buffer, loop until nothing in buffer.
    inchar = Serial1.read(); 
  } 
while (Serial1.available() > 0);
//==============================================================
// send command for data
      Serial1.println("S4");                  
      delay(Dlay);                            //allow web site to respond.
      tempval="";
       
// check for data arrival
  if (Serial1.available() > 0) 
      {
        // read the incoming byte.
        inchar = Serial1.read();                 //#-10.76%*
        if (inchar=='#')
           { 
              inchar = Serial1.read();          //+ or -
              if (inchar =='-')
                {
                  tft.setTextColor(TFT_RED, TFT_BLACK);
                }
              else
                {
                  tft.setTextColor(TFT_GREEN, TFT_BLACK);  
                }
              tft.drawChar(inchar, 160,162, 2);
              inchar = Serial1.read();          //1
              tft.drawChar(inchar, 170,162, 2);
              inchar = Serial1.read();          //0
              tft.drawChar(inchar, 180,162, 2);
              inchar = Serial1.read();          //.
              tft.drawChar(inchar, 185,162, 2);
              inchar = Serial1.read();          //7
              tft.drawChar(inchar, 195, 162, 2);
              inchar = Serial1.read();          //6
              tft.drawChar(inchar, 205,162, 2);
              inchar = Serial1.read();          //%
              if (inchar !='*'){tft.drawChar(inchar, 215,162, 2);}
              tft.setTextColor(TFT_WHITE, TFT_BLACK);
           }
      }
  else
      {
        // no respnse from web site.
        tft.drawChar('?',290,110, 2);
      } 
//----------------------------------------------------------
  min10Stock=1; 
  delay(2000); //wait for possible data
  do 
    {//empty serial buffer, loop until nothing in buffer.
      inchar = Serial1.read(); 
    } 
  while (Serial1.available() > 0);   
}

//---------------------------------------------------------------------
// Start Up - Update weather and time data.
//---------------------------------------------------------------------
void  StartUP()
{
  if (startup_min15_1==1)
    {
       if (min15PA==0)
        {
          tft.setTextColor(TFT_YELLOW, TFT_BLACK);    // Set color back to yellow
          tft.drawString("---- Weather Updating ----  ",80,124, 2);// Title
          tft.setTextColor(TFT_WHITE, TFT_BLACK);    // Set color back to white}  
          min15uptime=1;
          Check_Temp1();    
        }//15 min PA temp.
    }
    
  if (startup_min15_4==1)
    {
      if (startup_min15_1==0 and min15FL==0){Check_Temp2();}
    }  //15 min FL temp. 
  
  if (startup_min15_3==1)
    {
      if (startup_min15_4==0 and min15KY==0){Check_Temp3();}
    }  //15 min KY temp.
  
  if (startup_min15_2==1)
    {
      if (startup_min15_3==0 and min15PAW==0)
        {
          Check_Temp4(); //PA weather.
          tft.drawString("---- Weather Forecast ----",80,124, 2);// Title
         } 
    }
}
 
//---------------------------------------------------------------------
// Update every 15 minutes.
//---------------------------------------------------------------------
void Weather_Update_15min()
{
    if (t.min==0 or t.min==15 or t.min==30 or t.min==45 or touchit==1)
      {
        min15up=1;
        touchit=0;
      }

    if (min15up==1)
      {
        if (min15PA==0)
         {
            tft.setTextColor(TFT_YELLOW, TFT_BLACK);    // Set color back to yellow
            tft.drawString("PA--- Weather Updating ----  ",80,124, 2);// Title
            tft.setTextColor(TFT_WHITE, TFT_BLACK);    // Set color back to white} 
            min15uptime=1;
            Check_Temp1();
         }   //15 min PA temp.
        if (min15FL==0)
          {
            tft.setTextColor(TFT_YELLOW, TFT_BLACK);    // Set color back to yellow
            tft.drawString("FL--- Weather Updating ----  ",80,124, 2);// Title
            tft.setTextColor(TFT_WHITE, TFT_BLACK);    // Set color back to white} 
            Check_Temp2();   //15 min FL temp.
           }
        
        
        if (min15KY==0)
          {
            tft.setTextColor(TFT_YELLOW, TFT_BLACK);    // Set color back to yellow
            tft.drawString("KY--- Weather Updating ----  ",80,124, 2);// Title
            tft.setTextColor(TFT_WHITE, TFT_BLACK);    // Set color back to white} 
            Check_Temp3();
          }   //15 min KY temp.
          
        if (min15PAW==0)
        {
          tft.setTextColor(TFT_YELLOW, TFT_BLACK);    // Set color back to yellow
          tft.drawString("PAW-- Weather Updating ----  ",80,124, 2);// Title
          tft.setTextColor(TFT_WHITE, TFT_BLACK);    // Set color back to white} 
          Check_Temp4();  //PA weather.
          tft.drawString("---- Weather Forecast ----",80,124, 2);// Title
        }
    }
}

//---------------------------------------------------------------------
// Daily, update 2am.
//---------------------------------------------------------------------
void Time_Update_Daily_2am()
{
  if (startup_srt==1 or t.hour==stmeHR)
    {
      if (startup_min15_2==0 and srt==0)   
        {
          tft.setTextColor(TFT_YELLOW, TFT_BLACK);    // Set color back to yellow
          tft.drawString("--- SUN Time Updating  --- ",80,124, 2);// Title
          tft.setTextColor(TFT_WHITE, TFT_BLACK);    // Set color back to white}  
          min15uptime=1;
          Check_SunRise();//Sun Rise time.
        }  
    }
  if (startup_sst==1 or t.hour==stmeHR)
    {
      if (startup_srt==0 and sst==0)
        {
          Check_SunSet();  //Sun set time.
        }
    }
    
//---------------------------------------------------------------------
// Get Current MonthYr, Day, Day of Week and Set RTC.
// Sundays, update 2am.   
//---------------------------------------------------------------------
noTimeUpdate=0;
if (startup_time2==1 or t.hour==stmeHR) 
    {
      if (startup_sst==0 and stime2==0)
        {
          tft.setTextColor(TFT_GREEN, TFT_BLACK);    // Set color back to green
          tft.drawString("----  Updating Time  ----  ",80,124, 2);// Title
          Get_RTC_MonthYr();//Get Current MonthYr.
          tft.drawString("MY--  Updating Time  ---- ",80,124, 2);// Title
        }  
    }
  if (startup_time3==1 or t.hour==stmeHR)
    {
      if (startup_time2==0 and stime3==0)
        {
          Get_RTC_Day();
          tft.drawString("MYD-  Updating Time  -----",80,124, 2);// Title
        }  //Get Current Day.
    }
  if (startup_time4==1 or t.hour==stmeHR)
    {
      if (startup_time3==0 and stime4==0)
        {
          Get_RTC_DOW();
           tft.drawString("MYD-  Updating Time  DOW--",80,124, 2);// Title
        }  //Get Current Day of week.
    }
//---------------------------------------------------------------------
// Get Current Time and Set RTC.
// Sunday, update 2am to pickup DST.
//---------------------------------------------------------------------
if (startup_time1==1 or t.hour==stmeHR) 
    {
      if (startup_time4==0 and stime1==0)
        {
          Get_RTC_Time();
          tft.drawString("MYD   Updating Time  DOW-T",80,124, 2);// Title
        }  //Get Current time.
    }

//Set RTC.
  if (startup_RTC==1 or t.hour==stmeHR)
    {
      if (startup_time1==0 and DS3231==0)
        {
          if (noTimeUpdate==0){RTC_Set();}
          tft.setTextColor(TFT_WHITE, TFT_BLACK);    // Set color back to white
          tft.drawString("---- Weather Forecast ----   ",80,124, 2);// Title
         }
    }
}

//---------------------------------------------------------------------
// Get time from RTC
//---------------------------------------------------------------------
void GetTime()
  {
    DS3231_get(&t);
  //----------------------------------------------------
  //make 12 hour time 1-12 am/pm
  //----------------------------------------------------
    if (t.hour < 12 and t.hour > 0) //=am 1-11am
      {
        tampm=t.hour;
        ampm="AM";
      }
    else
      {
        if (t.hour > 12 and t.hour < 24) //=pm 12-23 
         {
            tampm=t.hour-12;
            ampm="PM";
          }
      }
    if (t.hour == 0) //=am 12 
      {
         tampm=12;
         ampm="AM";
      }
       if (t.hour == 12) //=pm 12 
      {
         tampm=12;
         ampm="PM";
      }
   }

//---------------------------------------------------------------------
// Display time on LCD
//---------------------------------------------------------------------
  void DisplayTime()
  {
    if (t.sec<2 or strt==1 or sett==1 or min15uptime==1 or min10uptime==1)
      {
        min15uptime=0;
        min10uptime=0;
    //-------------------------------------------------------------
    // hours (update when seconds at 0, or at startup of program)
    //-------------------------------------------------------------
    if (sett==1){sett=0;}
    strt=0;
    tft.setTextSize(1);
     if (tampm < 10)
      {
        tft.setTextColor(TFT_BLACK, TFT_BLACK);   // Set color to black
        tft.drawNumber(12, 20, 45, 8);            // clear 12
        tft.setTextColor(TFT_GREEN, TFT_BLACK);   // Set color back to green
        tft.drawNumber(tampm, 80,45, 8);          // display hours
        tft.drawChar(':', 130, 38, 8);
      }
    else
      { //hours = 10,11,12
        tft.setTextColor(TFT_GREEN, TFT_BLACK);     // Set color to green
        tft.drawNumber(tampm, 20,45, 8);            // display hours
        tft.drawChar(':', 130, 38, 8);
      }
  
    //-------------------------------------------------------------
    // minutes  
    //-------------------------------------------------------------
    // Reset for 10 min update (stocks)
    if (t.min==5 or t.min==15 or t.min==25 or t.min==35 or t.min==45 or t.min==55)
      {
        min10Stock=0;
         tmin=0; 
      }
      
    // Reset for 2am update (time RTC)
    if (startup_RTC==0 and t.hour<stmeHR) // 2 set, 1 clear flags
      {
        stime1=0; 
        stime2=0;
        stime3=0;
        stime4=0;
        srt=0;
        sst=0;
        DS3231=0;
      }
      
    // Reset for 15 min update (weather)
    if (t.min==5 or t.min==20 or t.min==35 or t.min==50)
      {
        min15PA=0;
        min15FL=0;
        min15KY=0;
        min15PAW=0;
        min15up=0;
       }
      
    if (t.min < 10)
      {
        tft.drawChar('0', 160, 45, 8);              // Add minutes leading zero
        tft.drawNumber(t.min, 215,45, 8);           // Draw minutes
      }
    else
      {
         tft.drawNumber(t.min, 160,45, 8);          // Draw minutes
      }
    
    //-------------------------------------------------------------
    // AM / PM
    //-------------------------------------------------------------
    tft.setTextColor(TFT_YELLOW, TFT_BLACK);          // Set color to yellow
    if (ampm=="PM"){tft.drawString("PM",273,84, 4); } // AM, PM
    if (ampm=="AM"){tft.drawString("AM",273,84, 4); } // AM, PM
   
    //-------------------------------------------------------------
    // Date
    //-------------------------------------------------------------
    tft.setTextColor(TFT_BLACK, TFT_BLACK);     // Set color to black
    tft.drawNumber(1, 12,18, 2);     // clear month 11.12  
     tft.setTextColor(TFT_YELLOW, TFT_BLACK);    // Set color to yellow
  // display month 1-12
    if (t.mon>9) //10-12
      {
        tft.drawNumber(t.mon, 12,18, 2);            // Add month
      }
    else
      {//t.mon <10 1-9
        tft.drawNumber(t.mon, 18,18, 2);
      }  
    tft.drawChar('/', 30, 18, 2);
    
    tft.setTextColor(TFT_BLACK, TFT_BLACK);     // Set color to black
    tft.fillRect(40, 18, 100, 15, TFT_BLACK);
    tft.setTextColor(TFT_YELLOW, TFT_BLACK);    // Set color to yellow
   // display day 1-31
    if (t.mday>9)//10-31
      {    
        tft.drawNumber(t.mday, 40,18, 2);           // Add day
        tft.drawChar('/', 60, 18, 2);             
        tft.drawNumber(t.year_s, 70,18, 2);         // Add year
      }
    else
      {//t.day <10 1-9 
        tft.setTextColor(TFT_YELLOW, TFT_BLACK);    // Set color to yellow
        tft.drawNumber(t.mday, 40,18, 2);
        tft.drawChar('/', 50, 18, 2);              
        tft.drawNumber(t.year_s, 60,18, 2);
      }
        
    //-------------------------------------------------------------
    // Day of Week
    //-------------------------------------------------------------
      tft.setTextSize(0);
      tft.setTextColor(TFT_BLACK, TFT_BLACK);           //Set color to black
      tft.fillRect(240, 18, 120, 20, TFT_BLACK);
      tft.setTextColor(TFT_YELLOW, TFT_BLACK);          //Set color to yellow
      if (t.wday==1){tft.drawString("Sunday",260,18, 2);}
      if (t.wday==2){tft.drawString("Monday",260,18, 2);}
      if (t.wday==3){tft.drawString("Tuesday",255,18, 2);}
      if (t.wday==4){tft.drawString("Wednesday",240,18, 2);}
      if (t.wday==5){tft.drawString("Thursday",250,18, 2);}
      if (t.wday==6){tft.drawString("Friday",265,18, 2);}
      if (t.wday==7){tft.drawString("Saturday",250,18, 2);}
    }// update at seconds=0
    
  //-------------------------------------------------------------
  // seconds (update every second)
  //-------------------------------------------------------------
  tft.setTextColor(TFT_GREEN, TFT_BLACK);     // Set color to green
  if (t.sec < 10)
    {
      tft.drawChar('0', 286, 60, 2);          // Add seconds leading zero
      tft.drawNumber(t.sec, 294,60, 2);       // Draw seconds
    }
  else
    {
      tft.drawNumber(t.sec, 286,60, 2);       // Draw seconds
    }
  }
  
//------------------------------------------------------------------------
void Check_Temp1() // PA Temp
// Every 15 min - Get PA, FL, KY temps and PA weather condition.
//------------------------------------------------------------------------
{             
   do 
      {//empty serial buffer, loop until nothing in buffer.
        inchar = Serial1.read(); 
      } 
    while (Serial1.available() > 0);   

// send command for data
        Serial1.println("W1");                  //get PA weather Temp.
        delay(Dlay);                            //allow web site to respond.
        tempval="";
         
// check for data arrival
    if (Serial1.available() > 0) 
        {
          // read the incoming byte.
          inchar = Serial1.read();                 //#100* or #-7*
          if (inchar=='#')
             {
                inchar = Serial1.read();          
                tempval += (char)inchar;
                inchar = Serial1.read();
                tempval += (char)inchar;
                inchar = Serial1.read();
                if (isDigit(inchar)){tempval += (char)inchar;}              
                tft.drawNumber(tempval.toInt(), 20,155, 6); //display temp.
                Curtemp=tempval.toInt();  // Save the temp
                PA_TempIs();              // annouce the temperature
              }
        }
    else
        {
          // no respnse from web site.
          tft.drawChar('?',290,110, 2);
         // tempval="-99";
        } 
//----------------------------------------------------------
 // Curtemp=tempval.toInt();
 // PA_TempIs();
  startup_min15_1=0;
  min15PA=1; 
  delay(2000); //wait for possible data
  do 
    {//empty serial buffer, loop until nothing in buffer.
      inchar = Serial1.read(); 
    } 
  while (Serial1.available() > 0);   
}
//------------------------------------------------------------------------
void Check_Temp2() // FL Temp
//------------------------------------------------------------------------
{
// send command for data
        Serial1.println("W4");                  //get FL weather Temp.
        delay(Dlay);                            //allow web site to respond.
        tempval="";
         
// check for data arrival
    if (Serial1.available() > 0) 
      {
        // read the incoming byte.
        inchar = Serial1.read();                 //#100*
        if (inchar=='#')
         {
            inchar = Serial1.read();             //FL Temp
            tempval += (char)inchar;
            inchar = Serial1.read();
            tempval += (char)inchar;
            inchar = Serial1.read();
            if (isDigit(inchar)){tempval += (char)inchar;}//FL Temp
            tft.drawNumber(tempval.toInt(), 134,155, 6); //display temp.
              }
        }
    else
        {
          // no respnse from web site.
          tft.drawChar('?',290,110, 2);
        } 
//----------------------------------------------------------
  startup_min15_4=0;
  min15FL=1; 
  delay(2000); //wait for possible data
  do 
    {//empty serial buffer, loop until nothing in buffer.
      inchar = Serial1.read(); 
    } 
  while (Serial1.available() > 0);   
}
//------------------------------------------------------------------------
void Check_Temp3() // KY Temp
//------------------------------------------------------------------------
{             
// send command for data
        Serial1.println("W3");                  //get KY weather Temp.
        delay(Dlay);                            //allow web site to respond.
        tempval="";
         
// check for data arrival
    if (Serial1.available() > 0) 
      {
        // read the incoming byte.
        inchar = Serial1.read();                 //#100*
        if (inchar=='#')
         {
            inchar = Serial1.read();             
            tempval += (char)inchar;
            inchar = Serial1.read();
            tempval += (char)inchar;
            inchar = Serial1.read();
            if (isDigit(inchar)){tempval += (char)inchar;}
            tft.drawNumber(tempval.toInt(), 244,155, 6); //display temp.
          }
       }
    else
      {
        // no respnse from web site.
        tft.drawChar('?',290,110, 2);
      } 
//----------------------------------------------------------
  startup_min15_3=0;
  min15KY=1; 
  delay(2000); //wait for possible data
  do 
    {//empty serial buffer, loop until nothing in buffer.
      inchar = Serial1.read(); 
    } 
  while (Serial1.available() > 0);
}
//------------------------------------------------------------------------
void Check_Temp4() // PA conditions
//------------------------------------------------------------------------
{
// send command for data
        Serial1.println("W2");                  //get PA weather Temp.
        delay(Dlay);                            //allow web site to respond.
        tempval="";
    
// check for data arrival
    if (Serial1.available() > 0) 
      {
        // read the incoming byte.
        inchar = Serial1.read();                
        if (inchar=='#')
         {
            inchar = Serial1.read();        
            if (inchar=='1'){tft.drawString("SUNNY            ",172,215, 2);}
            if (inchar=='2'){tft.drawString("RAINY            ",172,215, 2);}
            if (inchar=='3'){tft.drawString("CLOUDY           ",172,215, 2);}
            if (inchar=='4'){tft.drawString("WINDY            ",172,215, 2);}
            if (inchar=='5'){tft.drawString("STORMY           ",172,215, 2);}
            if (inchar=='6'){tft.drawString("FOG              ",172,215, 2);}
            if (inchar=='7'){tft.drawString("FAIR             ",172,215, 2);}
            if (inchar=='8'){tft.drawString("PARTLY CLOUDY    ",172,215, 2);}
            if (inchar=='9'){tft.drawString("MOSTLY CLOUDY    ",172,215, 2);}
            if (inchar=='A'){tft.drawString("SNOW             ",172,215, 2);}
            if (inchar=='B'){tft.drawString("SNOW SHOWERS     ",172,215, 2);}
            tft.drawChar(' ',290,110, 2);     
          }
       }
    else
      {
        // no respnse from web site.
        tft.drawChar('?',290,110, 2);
      } 
//----------------------------------------------------------
  startup_min15_2=0;
  min15PAW=1;
  delay(2000); //wait for possible data
  do 
    {//empty serial buffer, loop until nothing in buffer.
      inchar = Serial1.read(); 
    } 
  while (Serial1.available() > 0);
}

//-------------------------------------------------------------
void Check_SunRise()
//-------------------------------------------------------------
{             
// send command for data
        Serial1.println("T5");                  
        delay(Dlay);                            //allow web site to respond.
        tempval="";
 
// check for data arrival
    if (Serial1.available() > 0) 
      {
        // read the incoming byte.
        inchar = Serial1.read();                 
        if (inchar=='#')
         {
            inchar = Serial1.read();                
            tft.drawChar(inchar,74,200, 2);        //7
            inchar = Serial1.read();                
            tft.drawChar(inchar,86,200, 2);        //:
            inchar = Serial1.read();                
            tft.drawChar(inchar,92,200, 2);        //3
            inchar = Serial1.read();                
            tft.drawChar(inchar,102,200, 2);        //4
            tft.drawString(" am",112,200, 2);       // am
         }
       }
    else
      {
        // no respnse from web site.
        tft.drawChar('?',290,110, 2);
      } 
//----------------------------------------------------------
  startup_srt=0;
  srt=1;   
  delay(2000); //wait for possible data
  do 
    {//empty serial buffer, loop until nothing in buffer.
      inchar = Serial1.read(); 
    } 
  while (Serial1.available() > 0);
}
//-------------------------------------------------------------
void Check_SunSet()
//-------------------------------------------------------------
{             
 
// send command for data
        Serial1.println("T6");                  
        delay(Dlay);                            //allow web site to respond.
        tempval="";

// check for data arrival
    if (Serial1.available() > 0) 
      {
        // read the incoming byte.
        inchar = Serial1.read();                 
        if (inchar=='#')
         {
            inchar = Serial1.read();                
            tft.drawChar(inchar,74,215, 2);        //7
            inchar = Serial1.read();                
            tft.drawChar(inchar,86,215, 2);        //:
            inchar = Serial1.read();                
            tft.drawChar(inchar,92,215, 2);        //3
            inchar = Serial1.read();                
            tft.drawChar(inchar,102,215, 2);        //4
            tft.drawString(" pm",112,215, 2);       // pm
         }
       }
    else
      {
        // no respnse from web site.
        tft.drawChar('?',290,110, 2);
      } 
//----------------------------------------------------------
  startup_sst=0;
  sst=1;    
  delay(2000); //wait for possible data
  do 
    {//empty serial buffer, loop until nothing in buffer.
      inchar = Serial1.read(); 
    } 
  while (Serial1.available() > 0);
}

//-------------------------------------------------------------
void Get_RTC_Time()
//-------------------------------------------------------------
{
//get current time
// send command for data
        Serial1.println("T1");                  //get current time
        delay(Dlay);                            //allow web site to respond.

// check for data arrival
    if (Serial1.available() > 0) 
      {
        // read the incoming byte.
        t_hr="";
        inchar = Serial1.read();                    //#12:13:25 PM
        if (inchar=='#')                            //#3:09:06 PM
         {
//----------------------------------------------------------
//get hours
//----------------------------------------------------------
           inchar = Serial1.read();         
           t_hr += (char)inchar;        
           inchar = Serial1.read();         // # or :   
           if (isDigit(inchar))
            {
              t_hr += (char)inchar;
              inchar = Serial1.read();      //:
            }
           t.hour=t_hr.toInt();           
//----------------------------------------------------------
// get minutes 13  09
//----------------------------------------------------------
           t_min="";
           inchar = Serial1.read();         
           t_min += (char)inchar;        
           inchar = Serial1.read();       // # or :    
           if (isDigit(inchar))
            {
               t_min += (char)inchar;
               inchar = Serial1.read();   //:
            }          
           t.min=t_min.toInt();         

//----------------------------------------------------------
// get seconds 25  06
//----------------------------------------------------------
           t_sec="";
           inchar = Serial1.read();         
           t_sec += (char)inchar;        
           inchar = Serial1.read();       //# or :    
           if (isDigit(inchar))
            {
              t_sec += (char)inchar;
              inchar = Serial1.read();   //space
            }
           t.sec=t_sec.toInt();
//----------------------------------------------------------
//Get am/pm
//----------------------------------------------------------
           inchar = Serial1.read();         //A or P
           if (inchar=='P')
             {//PM
               if (t.hour < 12){t.hour= t.hour + 12;}
             }
           else
             {//AM
              if (t.hour == 12){t.hour= 0;}
              }
         }
         else
         {
           noTimeUpdate=1;
           tft.drawChar('?',290,110, 2);
         }
      }
    else
      {
        // no respnse from web site.
        noTimeUpdate=1;
        tft.drawChar('?',290,110, 2);
      } 
//----------------------------------------------------------
  startup_time1=0;
  stime1=1;    
  delay(2000); //wait for possible data
  do 
    {//empty serial buffer, loop until nothing in buffer.
      inchar = Serial1.read(); 
    } 
  while (Serial1.available() > 0);
}

//-------------------------------------------------------------
void Get_RTC_MonthYr()
//-------------------------------------------------------------
{
//set current MonthYr Dec 2022
// send command for data
        Serial1.println("T2");                  //get current month and year.
        delay(Dlay);                            //allow web site to respond.

// check for data arrival
    if (Serial1.available() > 0) 
      {
        // read the incoming byte.
        t_mon="";
        inchar = Serial1.read();                    //#Nov 2022
        if (inchar=='#')                            
         {
//----------------------------------------------------------
//get month 
// Jan,Feb,Mar,Apr,May,Jun,Jul,Aug,Sep,Oct,Nov,Dec
//----------------------------------------------------------
           inchar = Serial1.read();     
           t_mon += (char)inchar;        //J
           inchar = Serial1.read();       
           t_mon += (char)inchar;        //a
           inchar = Serial1.read();       
           t_mon += (char)inchar;        //n
           if (t_mon=="Jan"){t.mon=1;}
           if (t_mon=="Feb"){t.mon=2;}
           if (t_mon=="Mar"){t.mon=3;}
           if (t_mon=="Apr"){t.mon=4;}
           if (t_mon=="May"){t.mon=5;}
           if (t_mon=="Jun"){t.mon=6;}
           if (t_mon=="Jul"){t.mon=7;}
           if (t_mon=="Aug"){t.mon=8;}
           if (t_mon=="Sep"){t.mon=9;}
           if (t_mon=="Oct"){t.mon=10;}
           if (t_mon=="Nov"){t.mon=11;}
           if (t_mon=="Dec"){t.mon=12;}
           inchar = Serial1.read();     // space  
//----------------------------------------------------------
           t_yr="";
           inchar = Serial1.read();        
           t_yr += (char)inchar;        //2
           inchar = Serial1.read();       
           t_yr += (char)inchar;        //20
           inchar = Serial1.read();       
           t_yr += (char)inchar;        //202  
           inchar = Serial1.read();       
           t_yr += (char)inchar;        //2022    
           t.year=t_yr.toInt();          //int 2022
        }
         else
         {
           noTimeUpdate=1;
           tft.drawChar('?',290,110, 2);
         }
      }
    else
      {
        // no respnse from web site.
        noTimeUpdate=1;
        tft.drawChar('?',290,110, 2);
      } 
//----------------------------------------------------------
  startup_time2=0;
  stime2=1;    
  delay(2000); //wait for possible data
  do 
    {//empty serial buffer, loop until nothing in buffer.
      inchar = Serial1.read(); 
    } 
  while (Serial1.available() > 0);
}

//-------------------------------------------------------------
void Get_RTC_Day()
//-------------------------------------------------------------
{
//set current Day
// send command for data
        Serial1.println("T3");                  //get current Day.
        delay(Dlay);                            //allow web site to respond.
 
// check for data arrival
    if (Serial1.available() > 0) 
      {
        // read the incoming byte.
        t_day="";
        inchar = Serial1.read();                    //#12:13:25 PM
        if (inchar=='#')                            //#3:09:06 PM
         {
//----------------------------------------------------------
//get day 1-31 
//----------------------------------------------------------
           inchar = Serial1.read();     
           t_day += (char)inchar;        //#
           inchar = Serial1.read();       
           t_day += (char)inchar;        //#
           t.mday=t_day.toInt();         //int 2022   
       }
         else
         {
           noTimeUpdate=1;
           tft.drawChar('?',290,110, 2);
         }
      }
    else
      {
        // no respnse from web site.
        noTimeUpdate=1;
        tft.drawChar('?',290,110, 2);
      } 
//----------------------------------------------------------
  startup_time3=0;
  stime3=1;  
  delay(2000); //wait for possible data
  do 
    {//empty serial buffer, loop until nothing in buffer.
      inchar = Serial1.read(); 
    } 
  while (Serial1.available() > 0);
}
//-------------------------------------------------------------
void Get_RTC_DOW()
//-------------------------------------------------------------
{
//set current Day of Week
// send command for data
        Serial1.println("T4");                  //get current day of week.
        delay(Dlay);                            //allow web site to respond.
    
// check for data arrival
    if (Serial1.available() > 0) 
      {
        // read the incoming byte.
        t_day="";
        inchar = Serial1.read();                    //#12:13:25 PM
        if (inchar=='#')                            //#3:09:06 PM
         {
//----------------------------------------------------------
//get DOW Mon,Tue,Wed,Thu,Fri,Sat,Sun 
//----------------------------------------------------------
           inchar = Serial1.read();     
           t_dow += (char)inchar;        //M
           inchar = Serial1.read();       
           t_dow += (char)inchar;        //o
           inchar = Serial1.read();       
           t_dow += (char)inchar;        //n
           if (t_dow=="Sun"){t.wday=1;}
           if (t_dow=="Mon"){t.wday=2;}
           if (t_dow=="Tue"){t.wday=3;}
           if (t_dow=="Wed"){t.wday=4;}
           if (t_dow=="Thu"){t.wday=5;}
           if (t_dow=="Fri"){t.wday=6;}
           if (t_dow=="Sat"){t.wday=7;}
      }
         else
         {
           noTimeUpdate=1;
           tft.drawChar('?',290,110, 2);
         }
      }
    else
      {
        // no respnse from web site.
        noTimeUpdate=1;
        tft.drawChar('?',290,110, 2);
      } 
//----------------------------------------------------------
  startup_time4=0;
  stime4=1;  
  delay(2000); //wait for possible data
  do 
    {//empty serial buffer, loop until nothing in buffer.
      inchar = Serial1.read(); 
    } 
  while (Serial1.available() > 0);
}

// --------------------------------------------------------
// Set the RTC using internet values!
// --------------------------------------------------------
void RTC_Set()
{
  DS3231_set(t);
  startup_RTC=0; 
  DS3231=1;
}
// --------------------------------------------------------
